// Generated by CoffeeScript 1.6.2
var Stream, buffer, connect, crypto, each, exec, fs, misc, path, rimraf, util, _ref;

crypto = require('crypto');

fs = require('fs');

path = require('path');

if ((_ref = fs.exists) == null) {
  fs.exists = path.exists;
}

each = require('each');

util = require('util');

Stream = require('stream');

exec = require('superexec');

connect = require('superexec/lib/connect');

buffer = require('buffer');

rimraf = require('rimraf');

misc = module.exports = {
  string: {
    /*
    `string.hash(file, [algorithm], callback)`
    ------------------------------------------
    Output the hash of a supplied string in hexadecimal 
    form. The default algorithm to compute the hash is md5.
    */

    hash: function(data, algorithm) {
      if (arguments.length === 1) {
        algorithm = 'md5';
      }
      return crypto.createHash(algorithm).update(data).digest('hex');
    }
  },
  file: {
    stat: function(ssh, path, callback) {
      if (!ssh) {
        return fs.stat(path, function(err, stat) {
          return callback(err, stat);
        });
      } else {
        return ssh.sftp(function(err, sftp) {
          if (err) {
            return callback(err);
          }
          return sftp.stat(path, function(err, attr) {
            sftp.end();
            if (err && err.type === 'NO_SUCH_FILE') {
              err.code = 'ENOENT';
              return callback(err);
            }
            return callback(err, attr);
          });
        });
      }
    },
    /*
    `readFile(ssh, path, [options], callback)`
    -----------------------------------------
    */

    readFile: function(ssh, path, options, callback) {
      var _ref1;

      if (arguments.length === 3) {
        callback = options;
        options = {};
      }
      if ((_ref1 = options.encoding) == null) {
        options.encoding = 'utf8';
      }
      if (!path) {
        return callback(new Error("Invalid path '" + path + "'"));
      }
      if (!ssh) {
        return fs.readFile(path, options.encoding, function(err, content) {
          return callback(err, content);
        });
      } else {
        return ssh.sftp(function(err, sftp) {
          var data, finish, s;

          if (err) {
            return callback(err);
          }
          s = sftp.createReadStream(path, options);
          data = [];
          s.on('data', function(d) {
            return data.push(d.toString());
          });
          s.on('error', function(err) {
            return finish(err);
          });
          s.on('close', function() {
            return finish(null, data.join(''));
          });
          return finish = function(err, data) {
            sftp.end();
            return callback(null, data);
          };
        });
      }
    },
    /*
    `writeFile(ssh, path, content, [options], callback)`
    -----------------------------------------
    */

    writeFile: function(ssh, path, content, options, callback) {
      var chmod, chown, finish, write;

      if (arguments.length === 4) {
        callback = options;
        options = {};
      }
      if (!ssh) {
        write = function() {
          var stream;

          stream = fs.createWriteStream(path, options);
          if (typeof content === 'string' || buffer.Buffer.isBuffer(content)) {
            if (content) {
              stream.write(content);
            }
            stream.end();
          } else {
            content.pipe(stream);
          }
          stream.on('error', function(err) {
            return callback(err);
          });
          stream.on('end', function() {
            return s.destroy();
          });
          return stream.on('close', function() {
            return chown();
          });
        };
        chown = function() {
          if (!(options.uid || options.gid)) {
            return chmod();
          }
          return fs.chown(path, options.uid, options.gid, function(err) {
            if (err) {
              return callback(err);
            }
            return chmod();
          });
        };
        chmod = function() {
          if (!options.mode) {
            return finish();
          }
          return fs.chmod(path, options.mode, function(err) {
            return finish(err);
          });
        };
        finish = function(err) {
          return callback(err);
        };
        return write();
      } else {
        return ssh.sftp(function(err, sftp) {
          if (err) {
            return callback(err);
          }
          write = function() {
            var s;

            s = sftp.createWriteStream(path, options);
            if (typeof content === 'string' || buffer.Buffer.isBuffer(content)) {
              if (content) {
                s.write(content);
              }
              s.end();
            } else {
              content.pipe(s);
            }
            s.on('error', function(err) {
              return finish(err);
            });
            s.on('end', function() {
              return s.destroy();
            });
            return s.on('close', function() {
              return chown();
            });
          };
          chown = function() {
            if (!(options.uid || options.gid)) {
              return chmod();
            }
            return sftp.chown(path, options.uid, options.gid, function(err) {
              if (err) {
                return finish(err);
              }
              return chmod();
            });
          };
          chmod = function() {
            if (!options.mode) {
              return finish();
            }
            return sftp.chmod(path, options.mode, function(err) {
              return finish(err);
            });
          };
          finish = function(err) {
            sftp.end();
            return callback(err);
          };
          return write();
        });
      }
    },
    /*
    `mkdir(ssh, path, [options], callback)`
    -------------------------------------
    Note, if option is not a string, it is considered to be the permission mode.
    */

    mkdir: function(ssh, path, options, callback) {
      if (arguments.length === 3) {
        callback = options;
        options = 0x1ed;
      }
      if (typeof options !== 'object') {
        options = {
          mode: options
        };
      }
      if (options.permissions) {
        process.stderr.write('Deprecated, use mode instead of permissions');
        options.mode = options.permissions;
      }
      if (!ssh) {
        return fs.mkdir(path, options.mode, function(err) {
          return callback(err);
        });
      } else {
        return ssh.sftp(function(err, sftp) {
          var chmod, chown, finish, mkdir;

          if (err) {
            return callback(err);
          }
          mkdir = function() {
            return sftp.mkdir(path, options, function(err, attr) {
              if (err) {
                return finish(err);
              }
              return chown();
            });
          };
          chown = function() {
            if (!(options.uid || options.gid)) {
              return chmod();
            }
            return sftp.chown(path, options.uid, options.gid, function(err) {
              if (err) {
                return finish(err);
              }
              return chmod();
            });
          };
          chmod = function() {
            if (!options.mode) {
              return finish();
            }
            return sftp.chmod(path, options.mode, function(err) {
              return finish(err);
            });
          };
          finish = function(err) {
            sftp.end();
            return callback(err);
          };
          return mkdir();
        });
      }
    },
    /*
    `exists(ssh, path, callback)`
    -----------------------------
    
    `options`         Command options include:   
    
    *   `ssh`         SSH connection in case of a remote file path.  
    *   `path`        Path to test.   
    *   `callback`    Callback to return the result.   
    
    `callback`        Received parameters are:   
    
    *   `err`         Error object if any.   
    *   `exists`      True if the file exists.
    */

    exists: function(ssh, path, callback) {
      if (!ssh) {
        return fs.exists(path, function(exists) {
          return callback(null, exists);
        });
      } else {
        return ssh.sftp(function(err, sftp) {
          if (err) {
            return callback(err);
          }
          return sftp.stat(path, function(err, attr) {
            sftp.end();
            return callback(null, err ? false : true);
          });
        });
      }
    },
    /*
    `files.hash(file, [algorithm], callback)`
    -----------------------------------------
    Output the hash of a supplied file in hexadecimal 
    form. If the provided file is a directory, the returned hash 
    is the sum of all the hashs of the files it recursively 
    contains. The default algorithm to compute the hash is md5.
    
    Throw an error if file does not exist unless it is a directory.
    */

    hash: function(file, algorithm, callback) {
      var hashs;

      if (arguments.length === 2) {
        callback = algorithm;
        algorithm = 'md5';
      }
      hashs = [];
      return fs.stat(file, function(err, stat) {
        if ((err != null ? err.code : void 0) === 'ENOENT') {
          return callback(new Error("Does not exist: " + file));
        }
        if (err) {
          return callback(err);
        }
        if (stat.isDirectory()) {
          file += '/**';
        }
        return each().files(file).on('item', function(item, next) {
          var shasum;

          shasum = crypto.createHash(algorithm);
          return fs.ReadStream(item).on('data', function(data) {
            return shasum.update(data);
          }).on('error', function(err) {
            if (err.code === 'EISDIR') {
              return next();
            }
            return next(err);
          }).on('end', function() {
            hashs.push(shasum.digest('hex'));
            return next();
          });
        }).on('error', function(err) {
          return callback(err);
        }).on('end', function() {
          switch (hashs.length) {
            case 0:
              if (stat.isFile()) {
                return callback(new Error("Does not exist: " + file));
              } else {
                return callback(null, crypto.createHash(algorithm).update('').digest('hex'));
              }
              break;
            case 1:
              return callback(null, hashs[0]);
            default:
              hashs = crypto.createHash(algorithm).update(hashs.join('')).digest('hex');
              return callback(null, hashs);
          }
        });
      });
    },
    /*
    `files.compare(files, callback)`
    --------------------------------
    Compare the hash of multiple file. Return the file md5 
    if the file are the same or false otherwise.
    */

    compare: function(files, callback) {
      var result;

      if (files.length < 2) {
        return callback(new Error('Minimum of 2 files'));
      }
      result = null;
      return each(files).parallel(true).on('item', function(file, next) {
        return misc.file.hash(file, function(err, md5) {
          if (err) {
            return next(err);
          }
          if (result === null) {
            result = md5;
          } else if (result !== md5) {
            result = false;
          }
          return next();
        });
      }).on('error', function(err) {
        return callback(err);
      }).on('end', function() {
        return callback(null, result);
      });
    },
    /*
    remove(ssh, path, callback)
    ---------------------------
    Remove a file or directory
    */

    remove: function(ssh, path, callback) {
      var child;

      if (!ssh) {
        return rimraf(path, callback);
      } else {
        child = exec("rm -rdf " + path, {
          ssh: ssh
        });
        return child.on('exit', function(code) {
          return callback(null, code);
        });
      }
    }
  },
  ssh: {
    /*
    passwd(sftp, [user], callback)
    ----------------------
    Return information present in '/etc/passwd' and cache the 
    result in the provided ssh instance as "passwd".
    
    Result look like: 
        { root: {
            uid: '0',
            gid: '0',
            comment: 'root',
            home: '/root',
            shell: '/bin/bash' }, ... }
    */

    passwd: function(ssh, username, callback) {
      if (arguments.length === 3) {
        if (!username) {
          return callback(null, null);
        }
        return misc.ssh.passwd(ssh, function(err, users) {
          var user;

          if (err) {
            return err;
          }
          user = users[username];
          if (!user) {
            return new Error("User " + username + " does not exists");
          }
          return callback(null, user);
        });
      }
      callback = username;
      username = null;
      if (ssh.passwd) {
        return callback(null, ssh.passwd);
      }
      return misc.file.readFile(ssh, '/etc/passwd', function(err, lines) {
        var info, line, passwd, _i, _len, _ref1;

        if (err) {
          return callback(err);
        }
        passwd = [];
        _ref1 = lines.split('\n');
        for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
          line = _ref1[_i];
          info = /(.*)\:\w\:(.*)\:(.*)\:(.*)\:(.*)\:(.*)/.exec(line);
          if (!info) {
            continue;
          }
          passwd[info[1]] = {
            uid: parseInt(info[2]),
            gid: parseInt(info[3]),
            comment: info[4],
            home: info[5],
            shell: info[6]
          };
        }
        ssh.passwd = passwd;
        return callback(null, passwd);
      });
    },
    /*
    group(sftp, [group], callback)
    ----------------------
    Return information present in '/etc/group' and cache the 
    result in the provided ssh instance as "group".
    
    Result look like: 
        { root: {
            password: 'x'
            gid: 0,
            user_list: [] },
          bin: {
            password: 'x',
            gid: 1,
            user_list: ['bin','daemon'] } }
    */

    group: function(ssh, group, callback) {
      if (arguments.length === 3) {
        if (!group) {
          return callback(null, null);
        }
        return misc.ssh.group(ssh, function(err, groups) {
          if (err) {
            return err;
          }
          group = groups[group];
          if (!group) {
            return new Error("Group " + group + " does not exists");
          }
          return callback(null, group);
        });
      }
      callback = group;
      group = null;
      if (ssh.group) {
        return callback(null, ssh.group);
      }
      return misc.file.readFile(ssh, '/etc/group', function(err, lines) {
        var info, line, _i, _len, _ref1;

        if (err) {
          return callback(err);
        }
        group = [];
        _ref1 = lines.split('\n');
        for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
          line = _ref1[_i];
          info = /(.*)\:(.*)\:(.*)\:(.*)/.exec(line);
          if (!info) {
            continue;
          }
          group[info[1]] = {
            password: info[2],
            gid: parseInt(info[3]),
            user_list: info[4] ? info[4].split(',') : []
          };
        }
        ssh.group = group;
        return callback(null, group);
      });
    }
  },
  /*
  `isPortOpen(port, host, callback)`: Check if a port is already open
  */

  isPortOpen: function(port, host, callback) {
    if (arguments.length === 2) {
      callback = host;
      host = '127.0.0.1';
    }
    return exec("nc " + host + " " + port + " < /dev/null", function(err, stdout, stderr) {
      if (!err) {
        return callback(null, true);
      }
      if (err.code === 1) {
        return callback(null, false);
      }
      return callback(err);
    });
  },
  /*
  `merge([inverse], obj1, obj2, ...]`: Recursively merge objects
  --------------------------------------------------------------
  On matching keys, the last object take precedence over previous ones 
  unless the inverse arguments is provided as true. Only objects are 
  merge, arrays are overwritten.
  
  Enrich an existing object with a second one:
    obj1 = { a_key: 'a value', b_key: 'b value'}
    obj2 = { b_key: 'new b value'}
    result = misc.merge obj1, obj2
    assert.eql result, obj1
    assert.eql obj1.b_key, 'new b value'
  
  Create a new object from two objects:
    obj1 = { a_key: 'a value', b_key: 'b value'}
    obj2 = { b_key: 'new b value'}
    result = misc.merge {}, obj1, obj2
    assert.eql result.b_key, 'new b value'
  
  Using inverse:
    obj1 = { b_key: 'b value'}
    obj2 = { a_key: 'a value', b_key: 'new b value'}
    misc.merge true, obj1, obj2
    assert.eql obj1.a_key, 'a value'
    assert.eql obj1.b_key, 'b value'
  */

  merge: function() {
    var clone, copy, from, i, inverse, name, options, src, target, to, _i;

    target = arguments[0];
    from = 1;
    to = arguments.length;
    if (typeof target === 'boolean') {
      inverse = !!target;
      target = arguments[1];
      from = 2;
    }
    if (typeof target !== "object" && typeof target !== 'function') {
      target = {};
    }
    for (i = _i = from; from <= to ? _i < to : _i > to; i = from <= to ? ++_i : --_i) {
      if ((options = arguments[i]) !== null) {
        for (name in options) {
          src = target[name];
          copy = options[name];
          if (target === copy) {
            continue;
          }
          if ((copy != null) && typeof copy === 'object' && !Array.isArray(copy)) {
            clone = src && (src && typeof src === 'object' ? src : {});
            target[name] = misc.merge(false, clone, copy);
          } else if (copy !== void 0) {
            if (!(inverse && typeof target[name] !== 'undefined')) {
              target[name] = copy;
            }
          }
        }
      }
    }
    return target;
  },
  /*
  `options(options, callback)`
  ----------------------------
  Normalize options. An ssh connection if needed if the key "ssh" 
  hold a configuration object. The 'uid' and 'gid' fields will 
  be converted to integer if they match a username or a group.
  */

  options: function(options, callback) {
    if (!Array.isArray(options)) {
      options = [options];
    }
    return each(options).on('item', function(options, next) {
      var connection, gid, uid;

      if ((options["if"] != null) && !Array.isArray(options["if"])) {
        options["if"] = [options["if"]];
      }
      if ((options.if_exists != null) && !Array.isArray(options.if_exists)) {
        options.if_exists = [options.if_exists];
      }
      if ((options.not_if_exists != null) && !Array.isArray(options.not_if_exists)) {
        options.not_if_exists = [options.not_if_exists];
      }
      connection = function() {
        if (!options.ssh) {
          return next();
        }
        return connect(options.ssh, function(err, ssh) {
          if (err) {
            return next(err);
          }
          options.ssh = ssh;
          return uid();
        });
      };
      uid = function() {
        if (!options.uid) {
          return gid();
        }
        if (typeof options.uid === 'number' || /\d+/.test(options.uid)) {
          return gid();
        }
        return misc.ssh.passwd(options.ssh, options.uid, function(err, user) {
          var _ref1;

          options.uid = user.uid;
          if ((_ref1 = options.gid) == null) {
            options.gid = user.gid;
          }
          return gid();
        });
      };
      gid = function() {
        if (!options.gid) {
          return next();
        }
        if (typeof options.gid === 'number' || /\d+/.test(options.gid)) {
          return next();
        }
        return misc.ssh.group(options.ssh, options.gid, function(err, group) {
          if (group) {
            options.gid = group.gid;
          }
          return next();
        });
      };
      return connection();
    }).on('both', function(err) {
      return callback(err, options);
    });
  }
};
